@@ -250,16 +250,33 @@ Bool		 *replyReadyRet;
 
 	_IceAddReplyWait (iceConn, replyWait);
 
-
 	/*
 	 * Note that there are two different replyWaits.  The first is
 	 * the one passed into IceProcessMessages, and is the replyWait
 	 * for the message the client is blocking on.  The second is
 	 * the replyWait for the message currently being processed
 	 * by IceProcessMessages.  We call it "useThisReplyWait".
+	 *
+	 * Also, when two hosts communicate over an ICE connection and use
+	 * different major opcodes for a subprotocol, it is impossible
+	 * to use message replies unless we translate opcodes before
+	 * comparing them.
 	 */
+	
+	{
+	    int op;
 
-	useThisReplyWait = _IceSearchReplyWaits (iceConn, header->majorOpcode);
+	    if (header->majorOpcode == 0)
+	    {
+		op = 0;
+	    }
+	    else
+	    {
+		int idx = header->majorOpcode - iceConn->his_min_opcode;
+		op = iceConn->process_msg_info[idx].my_opcode;
+	    }
+	    useThisReplyWait = _IceSearchReplyWaits (iceConn, op);
+	}
     }
 
     if (header->majorOpcode == 0)
@@ -598,7 +615,7 @@ IceReplyWaitInfo *replyWait;
 	    _IceConnectionError *errorReply =
 	        &(((_IceReply *) (replyWait->reply))->connection_error);
 	    char *errorStr = NULL;
-	    char *tempstr;
+	    const char *tempstr;
 	    char *prefix, *temp;
 
 	    invokeHandler = 0;
@@ -610,16 +627,14 @@ IceReplyWaitInfo *replyWait;
 
 		tempstr =
 		    "None of the ICE versions specified are supported";
-		errorStr = (char *) malloc (strlen (tempstr) + 1);
-		strcpy (errorStr, tempstr);
+		errorStr = strdup(tempstr);
 		break;
 
 	    case IceNoAuth:
 
 		tempstr =
 		    "None of the authentication protocols specified are supported";
-		errorStr = (char *) malloc (strlen (tempstr) + 1);
-		strcpy (errorStr, tempstr);
+		errorStr = strdup(tempstr);
 		break;
 
 	    case IceSetupFailed:
@@ -680,16 +695,14 @@ IceReplyWaitInfo *replyWait;
 
 	        temp =
 		    "None of the protocol versions specified are supported";
-		errorStr = (char *) malloc (strlen (temp) + 1);
-		strcpy (errorStr, temp);
+		errorStr = strdup(temp);
 		break;
 
 	    case IceNoAuth:
 
 		temp =
 		    "None of the authentication protocols specified are supported";
-		errorStr = (char *) malloc (strlen (temp) + 1);
-		strcpy (errorStr, temp);
+		errorStr = strdup(temp);
 		break;
 
 	    case IceSetupFailed:
@@ -1112,11 +1125,11 @@ IceReplyWaitInfo	*replyWait;
 	    _IceConnectionError *errorReply =
 	        &(((_IceReply *) (replyWait->reply))->connection_error);
 
-	    char *tempstr = "Received bad authIndex in the AuthRequired message";
+	    const char *tempstr
+		= "Received bad authIndex in the AuthRequired message";
 	    char errIndex = (int) message->authIndex;
 
-	    errorString = (char *) malloc (strlen (tempstr) + 1);
-	    strcpy (errorString, tempstr);
+	    errorString = strdup(tempstr);
 
 	    errorReply->type = ICE_CONNECTION_ERROR;
 	    errorReply->error_message = errorString;
@@ -1142,11 +1155,11 @@ IceReplyWaitInfo	*replyWait;
 	    _IceProtocolError *errorReply =
 	        &(((_IceReply *) (replyWait->reply))->protocol_error);
 
-	    char *tempstr = "Received bad authIndex in the AuthRequired message";
+	    const char *tempstr
+		= "Received bad authIndex in the AuthRequired message";
 	    char errIndex = (int) message->authIndex;
 
-	    errorString = (char *) malloc (strlen (tempstr) + 1);
-	    strcpy (errorString, tempstr);
+	    errorString = strdup(tempstr);
 
 	    errorReply->type = ICE_PROTOCOL_ERROR;
 	    errorReply->error_message = errorString;
